indexing

	description:
		"Basic mathematical operations, double-precision. %
		%This class may be used as ancestor by classes needing its %
		%facilities."
	status: "See notice at end of class"
	date: "$Date: 1998/08/12 16:51:45 $"
	revision: "$Revision: 1.13 $"

class
	DOUBLE_MATH

inherit

	MATH_CONST
		export
			{NONE} all
		end

feature -- Access

	log_2 (v: DOUBLE): DOUBLE is
			-- Base 2 logarithm of `v'
		local
			a: DOUBLE
		do
			a := 2.0;
			Result := log (v) / log (a)
		end;

	cosine (v: DOUBLE): DOUBLE is
			-- Trigonometric cosine of radian `v' approximated
			-- in the range [-pi/4, +pi/4]
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"cos"
		end;

	arc_cosine (v: DOUBLE): DOUBLE is
			-- Trigonometric arccosine of radian `v'
			-- in the range [0, pi]
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"acos"
		end;

	sine (v: DOUBLE): DOUBLE is
			-- Trigonometric sine of radian `v' approximated
			-- in range [-pi/4, +pi/4]
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"sin"
		end;

	arc_sine (v: DOUBLE): DOUBLE is
			-- Trigonometric arcsine of radian `v'
			-- in the range [-pi/2, +pi/2]
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"asin"
		end;

	tangent (v: DOUBLE): DOUBLE is
			-- Trigonometric tangent of radian `v' approximated
			-- in range [-pi/4, +pi/4]
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"tan"
		end;

	arc_tangent (v: DOUBLE): DOUBLE is
			-- Trigonometric arctangent of radian `v'
			-- in the range [-pi/2, +pi/2]
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"atan"
		end;

	sqrt (v: DOUBLE): DOUBLE is
			-- Square root of `v'
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		end;

	log (v: DOUBLE): DOUBLE is
			-- Natural logarithm of `v'
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		end;

	log10 (v: DOUBLE): DOUBLE is
			-- Base 10 logarithm of `v'
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		end;

	floor (v: DOUBLE): DOUBLE is
			-- Greatest integral less than or equal to `v'
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		end;

	ceiling (v: DOUBLE): DOUBLE is
			-- Least integral greater than or equal to `v'
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"ceil"
		end;

feature {NONE} -- Implementation

	dabs (v: DOUBLE): DOUBLE is
			-- Absolute of `v'
		external
--			"C | <math.h>"
			"C_WithoutCurrent"
		alias
			"fabs"
		end;

end -- class DOUBLE_MATH


--|----------------------------------------------------------------
--| EiffelBase: Library of reusable components for Eiffel.
--| Copyright (C) 1986-1998 Interactive Software Engineering (ISE).
--| For ISE customers the original versions are an ISE product
--| covered by the ISE Eiffel license and support agreements.
--| EiffelBase may now be used by anyone as FREE SOFTWARE to
--| develop any product, public-domain or commercial, without
--| payment to ISE, under the terms of the ISE Free Eiffel Library
--| License (IFELL) at http://eiffel.com/products/base/license.html.
--|
--| Interactive Software Engineering Inc.
--| ISE Building, 2nd floor
--| 270 Storke Road, Goleta, CA 93117 USA
--| Telephone 805-685-1006, Fax 805-685-6869
--| Electronic mail <info@eiffel.com>
--| Customer support e-mail <support@eiffel.com>
--| For latest info see award-winning pages: http://eiffel.com
--|----------------------------------------------------------------

